// Fossil C++ implementation
// V1.1 - Giovanni Iachello 1992 - Public Domain
// Pubblicato nel numero 28 di Telematicus

#ifndef _FOSSIL_HPP_
#define _FOSSIL_HPP_

#include <stdio.h>
#include <dos.h>

#define FOSSILE_INT 0x14

#define COM1 0
#define COM2 1
#define COM3 2
#define COM4 3
#define TEST 0x00ff;

#define XONXOFF_TX 1
#define CTSRTS 2
#define XONXOFF_RX 8

typedef unsigned short BOOL;
#define	FALSE 0
#define	TRUE 1


enum BAUDRATE {
	BR19200 = 0,
	BR38400 = 1,
	BR300 = 2,
	BR600 = 3,
	BR1200 = 4,
	BR2400 = 5,
	BR4800 = 6,
	BR9600 = 7
};

enum PARITY {
	PANONE = 0,
	PAODD = 1,
	PAEVEN = 3
};

enum STOPBITS {
	SB1 = 0,
	SB2 = 1
};

enum CHARLENGTH {
	CL5 = 0,
	CL6,
	CL7,
	CL8
};


struct StatusBits {
	unsigned : 3;
	unsigned NULLCD: 1;
	unsigned : 3;
	unsigned DCD: 1;
	unsigned RDA : 1;
	unsigned OVRN: 1;
	unsigned : 3;
	unsigned THRE: 1;
	unsigned TSRE: 1;
	unsigned : 1;
};

// questa pragma forza il byte alignment sulla seguente struttura...
#pragma option -a-

struct DriverInfo {
	unsigned int size;
	unsigned char version;
	unsigned char revision;
	char far *ID ;
	unsigned int inbufsize;
	unsigned int inbuffree;
	unsigned int outbufsize;
	unsigned int outbuffree;
	unsigned char screenwidth;
	unsigned char screenheight;
	unsigned char baud;
};

class FossilDriver {
private:
	void interrupt (*intvect)(...);

	BOOL status;              // FALSE if init failed
	StatusBits statbits;
	unsigned int port;
	unsigned char revision;
	unsigned char maxfunc;
	unsigned char last;
	unsigned char timerIntNo;
	unsigned char ticksxsec;
	unsigned int msxtick;


	DriverInfo driverinfo;

	void CheckCall(unsigned char call)
	{
		if (call>maxfunc) printf("Function %x not supported by Fossil driver!\n",(int)call);
	}

	void GetSystemTimerParams(void);
	void Reboot(unsigned int type);

	BOOL ctrlCflag;
public:
	FossilDriver (unsigned int port);
	~FossilDriver(void);

	StatusBits SetBaudRate(BAUDRATE br,PARITY pa=PANONE, STOPBITS sb=SB1, CHARLENGTH cl=CL8);
	StatusBits TxCharWait(unsigned char c);
	BOOL TxCharNoWait(unsigned char c);
	unsigned char RxCharWait(void);
	unsigned char PeekInChar(void);
	BOOL IsInChar(void);
	void RaiseDTR(void);
	void LowerDTR(void);
	void FlushOutputBuffer(void);
	void PurgeOutputBuffer(void);
	void PurgeInputBuffer(void);
	BOOL SetTimerFunc(void (far * func)(),BOOL status);
	unsigned int ReadBlock(char far* buffer,unsigned int count);
	unsigned int WriteBlock(char far* buffer,unsigned int count);

	StatusBits StatusRequest(void);
	BOOL Initialize(void);
	void Deinitialize(void);
	void StartBreak(void);
	void StopBreak(void);

	void SetFlowControl(unsigned char m);
	void EnableXonXoffTx(void) { SetFlowControl(XONXOFF_TX); }
	void EnableXonXoffRx(void) { SetFlowControl(XONXOFF_RX); }
	void EnableCTSRTS(void) { SetFlowControl(CTSRTS); }

	void GetDriverInformation(void);
	BOOL SetCtrlCK(BOOL status);
	BOOL SetTransmitter(BOOL status);

	void SetWatchdog(BOOL status);

	BOOL GetStatus(void) { return status; };
	unsigned int GetPort(void) {return port; } ;
	unsigned char GetRevision(void) {return revision; };
	unsigned char GetMaxfunc(void) {return maxfunc;} ;
	unsigned char GetTimerIntNo(void) {return timerIntNo;} ;
	unsigned char GetTicksXSec(void) {return ticksxsec;} ;
	unsigned int GetMsXTick(void) {return msxtick;} ;

	DriverInfo& GetDriverInfo(void) { return driverinfo; }

	unsigned char ReadKeybWait(void);
	unsigned char PeekKeybChar(void);
	BOOL IsKeybChar(void);

	void SetCursor(unsigned int row, unsigned int col);
	int GetCursorRow(void);
	int GetCursorCol(void);

	void WriteAnsiChar(unsigned char c);
	void WriteChar(unsigned char c);

	void WarmBoot(void);
	void ColdBoot(void);

};

#endif

